/*-
 * Copyright 2016 UPLEX - Nils Goroll Systemoptimierung
 * All rights reserved.
 *
 * Author: Geoffrey Simmons <geoffrey.simmons@uplex.de>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include "config.h"

#include "vcl.h"
#include "vrt.h"
#include "vas.h"
#include "vdef.h"
#include "cache/cache.h"
#include "vsb.h"
#include "vsha256.h"

#include "vcc_if.h"
#include "parse_algorithm.h"

#define ERR(ctx, msg) \
	errmsg((ctx), "vmod blobcode error: " msg)

#define ERRNOMEM(ctx, msg) \
	ERR((ctx), msg ", out of space")

#define INIT_FINI(ctx) (((ctx)->method & (VCL_MET_INIT | VCL_MET_FINI)) != 0)

#define ILLEGAL(ctx, m) \
	ERR((ctx), m " is illegal in vcl_init() and vcl_fini().")

static void
errmsg(VRT_CTX, const char *fmt, ...)
{
	va_list args;

	va_start(args, fmt);
	if (ctx->method == VCL_MET_INIT) {
		AN(ctx->msg);
		VSB_vprintf(ctx->msg, fmt, args);
		VRT_handling(ctx, VCL_RET_FAIL);
	}
	else if (ctx->vsl)
		VSLbv(ctx->vsl, SLT_VCL_Error, fmt, args);
	else
		/* Should this ever happen in vcl_fini() ... */
		VSL(SLT_VCL_Error, 0, fmt, args);
	va_end(args);
}

VCL_BLOB
vmod_hash(VRT_CTX, VCL_ENUM hashs, VCL_BLOB msg)
{
	/* enum algorithm hash = parse_algorithm(hashs); */
	struct vmod_priv *b;
	char *snap;
	SHA256_CTX hashctx[1];
	(void) hashs;

	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
	if (INIT_FINI(ctx)) {
		ILLEGAL(ctx, "hash()");
		return NULL;
	}
	if (msg == NULL)
		return NULL;

	CHECK_OBJ_NOTNULL(ctx->ws, WS_MAGIC);
	snap = WS_Snapshot(ctx->ws);
	if ((b = WS_Alloc(ctx->ws, sizeof(struct vmod_priv))) == NULL) {
		ERRNOMEM(ctx, "allocating blob in hash()");
		return NULL;
	}
	if ((b->priv = WS_Alloc(ctx->ws, SHA256_LEN)) == NULL) {
		WS_Reset(ctx->ws, snap);
		ERRNOMEM(ctx, "allocating hash result in hash()");
		return NULL;
	}
	b->len = SHA256_LEN;
	b->free = NULL;
	SHA256_Init(hashctx);
	SHA256_Update(hashctx, msg->priv, msg->len);
	SHA256_Final(b->priv, hashctx);
	return b;
}

VCL_STRING
vmod_version(VRT_CTX __attribute__((unused)))
{
	return VERSION;
}
